<?php


/**
 * Check authorization in the http request header and age of timestamp
 * On a failed check the request will be terminated
 * @global int $iMaxAge         max allowed age
 * @param type $sMySecret
 * @return boolean
 */
function _checkAuth($sMySecret){
    global $iMaxAge;
    $aReqHeaders=apache_request_headers();
    _wd('request headers: <pre>'.print_r($aReqHeaders, 1).'</pre>');
    if(!isset($aReqHeaders['Authorization'])){
        _quit('Access denied. Missing authorization.', 403);
    }         
    if(!isset($aReqHeaders['Date'])){
        _quit('Access denied. Missing field "Date:" in the request header.', 403);
    }         

    $sGotHash= preg_replace('/^.*\:/', '', $aReqHeaders['Authorization']);
    $sGotDate= $aReqHeaders['Date'];
    $sGotMethod=$_SERVER['REQUEST_METHOD'];
    $sGotReq=$_SERVER['REQUEST_URI'];


    $sMyData="${sGotMethod}\n${sGotReq}\n${sGotDate}\n";
    $sMyHash= base64_encode(hash_hmac("sha1", $sMyData, $sMySecret));

    _wd('Hash: '.$sGotHash.' -- from header');
    _wd('Hash: '.$sMyHash.' -- rebuilt');
    if($sGotHash!==$sMyHash){
        _quit('Access denied. Invalid hash.', 403);
    }

    $iAge=date('U')-date('U', strtotime($sGotDate));
    _wd('Date: '.$sGotDate.' - age: '.$iAge.' sec');
    if($iAge>$iMaxAge){
        _quit('Access denied. Hash is out of date: '.$iAge. ' sec is older '.$iMaxAge.' sec. Maybe client or server is out of sync.', 403);
    }
    if($iAge<-$iMaxAge){
        _quit('Access denied. Hash is '.$iAge. ' sec in future but only '.$iMaxAge.' sec are allowed. Maybe client or server is out of sync.', 403);
    }
    return true;
}

/**
 * end with OK output
 * @param type $Data
 */
function _done($Data){
    echo is_array($Data) 
        ? json_encode($Data, JSON_PRETTY_PRINT)
        : $Data
        ;
    die();
}

/**
 * abort execution with error
 * @param string   $s        message
 * @param integer  $iStatus  http status code to send
 */
function _quit($s, $iStatus=400){
    $aStatus=array(
       400=>'HTTP/1.0 400 Bad Request', 
       403=>'HTTP/1.0 403 Access denied', 
       404=>'HTTP/1.0 404 Not found', 
    );
    header($aStatus[$iStatus]);
    # _done(array('status'=>$iStatus, 'info'=>$aStatus[$iStatus], 'message'=>$s));
    _sendHtml($aStatus[$iStatus], $s);
    die();
}

/**
 * send html page
 * @param type $sTitle
 * @param type $sContent
 */
function _sendHtml($sTitle, $sContent){
    echo '<!doctype html>
<html>
    <head>
        <title>'.$sTitle.'</title>
        <link rel="stylesheet" type="text/css" href="/main.css" media="screen" />
    </head>
    <body>
        <h1><span class="imllogo"></span> CI <span class="subdomain"> packages</span></h1>
        <h2>'.$sTitle.'</h2>

        <p>
            '.$sContent.'
        </p>

        <footer>
            <a href="https://www.iml.unibe.ch/">www.iml.unibe.ch</a>
        </footer>
    </body>
</html>
'
;
}

/**
 * write debug text (if enabled)
 * @global boolean $bDebug
 * @param string  $s       message
 * @param string  $sLevel  level; one of info|
 * @return boolean
 */
function _wd($s, $sLevel='info'){
    global $bDebug;
    if ($bDebug){
        echo '<div class="debug debug-'.$sLevel.'">DEBUG: '.$s.'</div>';
    }
    return true;
}

